#include "UserDb.h"
#define DEBUG
using namespace userdb;

UserDb::UserDb(const char *filename) : filename(filename)
{
}

bool UserDb::remove_user(User &user)
{
    return remove_user_by_line(filename, user.line);
}
bool UserDb::add_user(User &user)
{

    File db = LittleFS.open(filename, "a");
    if (db)
    {
        if (db.size() > 0)
            db.write("\n");
        String csvline = user.toCSVString();
        db.write(csvline.c_str());
        db.close();
        return true;
    }
    else
        return false;
}

User UserDb::user_by_pin(String cmp)
{
#ifdef DEBUG
    Serial.println("Searching for user with pin number " + cmp);
#endif
    Iterator it = begin_with_filter(&cmp, USERATTRIBUTES::USER_PIN);
    do
    {
        User temp = *it;
        if (temp.match)
            return temp;
        ++it;
    } while (it.has_next());
    return User{.enabled = false};
}
User UserDb::user_by_rfid(String cmp)
{
#ifdef DEBUG
    Serial.println("Searching for user with rfid " + cmp);
#endif
    Iterator it = begin_with_filter(&cmp, USERATTRIBUTES::RFID_UID);
    do
    {
        User temp = *it;
        if (temp.match)
            return temp;
        ++it;
    } while (it.has_next());
    return User{.enabled = false};
}

User UserDb::user_by_uid(unsigned long cmp)
{
    String cmps = String(cmp);
    return user_by_uid(cmps);
}
User UserDb::user_by_uid(String cmp)
{
#ifdef DEBUG
    Serial.println("Searching for user with id " + cmp);
#endif
    Iterator it = begin_with_filter(&cmp, USERATTRIBUTES::USER_ID);
    do
    {
        User temp = *it;
        if (temp.match)
            return temp;
        ++it;
    } while (it.has_next());
    return User{.enabled = false};
}
void UserDb::print_to_serial()
{
    Serial.println("Starting UserDB Test Read");
    for (Iterator it = begin(); it.has_next(); ++it)
    {
        User temp = *it;
        Serial.println(temp.toString());
    }
}
