#include "WebConsole.h"

using namespace webconsole;

WebConsole::WebConsole() {}
WebConsole::~WebConsole()
{
    _server->close();
    delete (_server);
}
bool WebConsole::init(userdb::UserDb *userdb)
{
    _server = new ESP8266WebServer(80);
    this->userdb = userdb;
    _server->begin();
    _server->on("/userdb", HTTPMethod::HTTP_DELETE, std::bind(&WebConsole::_dropUserDb, this));
    _server->on("/userdb", HTTPMethod::HTTP_GET, std::bind(&WebConsole::_getUserDb, this));
    _server->on("/rfid", std::bind(&WebConsole::_catchRFID, this));
    _server->on(UriBraces("/user/{}"), HTTPMethod::HTTP_DELETE, std::bind(&WebConsole::_deleteUser, this));
    _server->on(UriBraces("/user/{}"), HTTPMethod::HTTP_GET, std::bind(&WebConsole::_getUser, this));
    _server->on(UriBraces("/user/{}"), HTTPMethod::HTTP_PUT, std::bind(&WebConsole::_createUser, this));
    _server->on(UriBraces("/get/{}"), std::bind(&WebConsole::_deleteUser, this));
    //_server->on("/bypin",std::bind(&WebConsole::_findPin,this));
    _server->onNotFound(std::bind(&WebConsole::_handleStatic, this));
    return true;
}
void WebConsole::attachRfid(Rfid *rfid){
    this->rfid=rfid;
}
void WebConsole::serve()
{
    if (catch_rfid && rfid != nullptr && rfid->available())
    {
        rfid_buffer = rfid->getID();
        catch_rfid_updated = true;
        catch_rfid = false;
    }
    _server->handleClient();
}
bool WebConsole::isInterceptingRfid(){
    return catch_rfid;
}
void WebConsole::_handleStatic()
{
    String path = path_prefix + _server->uri();
    Serial.print("Request " + path);
    File src = LittleFS.open(path, "r");
    if (src.isDirectory())
    {
        path += "index.html";
        src.close();
        src = LittleFS.open(path, "r");
    }
    if (!src)
    {
        path = "/static/error404.html";
        src.close();
        src = LittleFS.open(path, "r");
    }

    Serial.println(" resolved to " + path);
    if (src)
    {
        String content_type = "text/html";
        _server->streamFile(src, content_type);
        src.close();
    }
    else
    {
        _server->send(500, "text/plain", "Internal error 500");
    }
}

void WebConsole::_getUserDb()
{
    File src = LittleFS.open("userdb.csv", "r");
    if (src)
    {
        _server->streamFile(src, "text/csv");
    }
    src.close();
}
void WebConsole::_deleteUser()
{
    if (userdb == nullptr)
    {
        _server->send(500, "text/json", "{\"error\":\"UserDb not initialized\"}");
        return;
    }
    String uid = _server->pathArg(0);
    userdb::User del = userdb->user_by_uid(uid);
    if (del.match == true)
    {
        userdb->remove_user(del);
        _server->send(200, "text/plain", del.toJSONString());
    }
    else
    {
        _server->send(500, "text/json", "{\"error\":\"User not found.\"}");
    }
}
void WebConsole::_getUser()
{
    if (userdb == nullptr)
    {
        _server->send(500, "text/json", "{\"error\":\"UserDb not initialized\"}");
        return;
    }
    String uid = _server->pathArg(0);
    userdb::User res = userdb->user_by_uid(uid);
    if (res.match == true)
    {
        _server->send(200, "text/json", res.toJSONString());
    }
    else
    {
        _server->send(500, "text/json", "{\"error\":\"User not found.\"}");
    }
}
void WebConsole::_createUser()
{
    userdb::User created;
    String body = _server->arg("plain");
    const int capacity = 1024;
    StaticJsonDocument<capacity> doc;
    DeserializationError err = deserializeJson(doc, body);
    if (err.code() != err.Ok)
    {
        Serial.println(err.c_str());
        Serial.println(body);
        String response = "{\"error\":\"";
        response += err.c_str();
        response += "\"}";
        _server->send(500, "text/json", response);
        return;
    }
    if (!_server->pathArg(0).isEmpty())
        created.uid = _server->pathArg(0).toInt();
    else if (!doc["uid"].isNull())
        created.uid = doc["uid"].as<unsigned long>();
    else
    {
        _server->send(500, "text/json", "{\"error\":\"UserId not provided.\"}");
        return;
    }
    created.first_name = doc["first_name"].as<String>();
    created.last_name = doc["last_name"].as<String>();
    created.rfid_uid = doc["rfid_uid"].as<String>();
    created.user_pin = doc["user_pin"].as<String>();
    created.enabled = doc["enabled"].as<bool>();
    userdb->add_user(created);
    _server->send(200, "text/json", created.toJSONString());
}
void WebConsole::_dropUserDb()
{
    if (userdb->drop())
        _server->send(500, "text/json", "{\"ok\":\"UserDb dropped.\"}");
    else
        _server->send(500, "text/json", "{\"error\":\"UserDb could  not be dropped.\"}");
}
void WebConsole::_catchRFID()
{
    if (catch_rfid_updated)
    {
        String response = "{\"rfid_uid\":\"" + rfid_buffer + "\"}";
        _server->send(500, "text/json", response);
        catch_rfid_updated = false;
    }
    else if(catch_rfid){
        _server->send(500, "text/json", "{\"ok\":\"already activated\"}");
    }
    else{
        catch_rfid = true;
        _server->send(500, "text/json", "{\"ok\":\"now activated\"}");
    }
}
